package com.huawei.dli.flink.demo;

import java.util.Properties;

import com.huawei.dli.flink.demo.avro.AvroDerializationSchema;
import com.huawei.dli.flink.demo.sink.*;
import com.huawei.dli.flink.demo.source.KafkaSource;
import com.huawei.dli.flink.demo.source.RandomSource;
import com.huawei.dli.flink.demo.utils.Log4jPrintStream;
import com.huawei.hwclouds.drs.avro.Record;
import org.apache.flink.api.common.serialization.DeserializationSchema;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.formats.json.JsonRowDeserializationSchema;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.streaming.api.functions.source.SourceFunction;
import org.apache.flink.types.Row;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 参数说明：
 *   --source source类型，当前支持kafka/rawRandom，默认为kafka
 *     * kafka source参数:
 *        --source.topic kafka源topic
 *        --bootstrap.servers kafka连接地址, 端口需为安全端口，可从Kafka客户端consumer.properties中获取
 *        --group.id 消费组
 *        --encode 源数据编码, 默认为raw string, 支持json/DSR avro
 *        --json.schema 输入源数据的json schame, encode为json时可配置
 *        --kerberos.domain.name kafka认证域名，可从Kafka客户端consumer.properties中获取
 *        --userPrincipal 创建的MRS机机用户名
 *     * rawRandom source参数:
 *        --sendInterval 发送数据周期
 *   --sink sink类型，当前支持hdfs/hbase/kafka/DSC/DWS
 *     * hdfs sink参数:
 *        --basePath 数据存储目录，需配置为完整的hdfs路径hdfs://${ip}:9820/${dir}，ip为namenode节点ip，端口为hdfs rpc端口
 *        --rolloverInterval 输出文件翻滚周期，默认为1分钟
 *     * dcs sink参数:
 *        --cluster.address dcs集群连接地址，格式为IP:PORT
 *        --password 认证密码
 *     * dws sink参数:
 *        --db.url 数据库连接地址，格式为jdbc:postgresql://${IP}:${PORT}/${dbName}
 *        --userName 数据库用户名
 *        --password 数据库密码
 *        --tableName 待写入数据表名，格式为${schema}.${table}，需提前创建: create table ${tableName}(${column} varchar);
 *     * 安全hive sink参数:
 *        --userPrincipal 创建的MRS机机用户名
 *        --zk.server.principal zookeeper server凭证，格式为: zookeeper/hadoop.${default_realm}
 *        --dbName 连接的数据库名, 默认为default
 */
public class Main {

  private static final Logger logger = LoggerFactory.getLogger(Main.class);

  public static void main(String[] args) throws Exception {
    Log4jPrintStream.redirectSystemOut();

    logger.info("USAGE:");
    logger.info("  1. kafka -> hdfs: --source.topic xx --bootstrap.servers xx:21007 --sink hdfs --basePath hdfs://xx:9820/data --kerberos.domain.name xx --userPrincipal xx");
    logger.info("  2. kafka -> hbase: --source.topic xx --bootstrap.servers xx:21007 --sink hbase --kerberos.domain.name xx --userPrincipal xx");

    ParameterTool params = ParameterTool.fromArgs(args);
    final StreamExecutionEnvironment env = StreamExecutionEnvironment
      .getExecutionEnvironment();
    // 机机账号用户名
    String userPrincipal = params.getRequired("userPrincipal");

    // Prepare stream source function
    String encode = params.get("encode");
    String sourceType = params.get("source", "kafka");
    SourceFunction<String> simpleSourceFunction = null;
    SourceFunction<Row> jsonSourceFunction = null;
    SourceFunction<Record> drsSourceFunction = null;
    String sourceName = null;
    switch (sourceType) {
      case "kafka":
        sourceName = "sec_kafka_source";

        Properties kafkaProps = new Properties();
        kafkaProps.setProperty("bootstrap.servers", params.getRequired("bootstrap.servers"));
        kafkaProps.setProperty("group.id", params.get("group.id", "test-group"));
        kafkaProps.setProperty("enable.auto.commit", "true");
        kafkaProps.setProperty("auto.offset.reset", "earliest");
        kafkaProps.setProperty("security.protocol", "SASL_PLAINTEXT");
        kafkaProps.setProperty("sasl.kerberos.service.name", "kafka");
        kafkaProps.setProperty("kerberos.domain.name", params.getRequired("kerberos.domain.name"));
        String topic = params.getRequired("source.topic");

        if (encode == null || encode.isEmpty()) {
          simpleSourceFunction = new KafkaSource<>(topic, new SimpleStringSchema(), kafkaProps, userPrincipal);
        } else if ("json".equalsIgnoreCase(encode)) {
          String jsonSchema = params.get("json.schema", "{\"type\":\"object\"," +
            "\"properties\":{\"id\":{\"type\":\"string\"},\"name\":{\"type\":\"string\"}," +
            "\"age\":{\"type\":\"integer\"}}}");
          jsonSourceFunction = new KafkaSource<>(
            topic, new JsonRowDeserializationSchema(jsonSchema), kafkaProps, userPrincipal);
        } else if ("drs_avro".equalsIgnoreCase(encode)) {
          DeserializationSchema<Record> valueDeserializer = new AvroDerializationSchema();
          drsSourceFunction = new KafkaSource<>(topic, valueDeserializer, kafkaProps, userPrincipal);
        } else {
          throw new IllegalArgumentException("Unsupported encode: " + encode);
        }
        break;
      case "rawRandom":
        // data stream with random numbers
        sourceName = "raw_random_source";

        long sendInterval = params.getLong("sendInterval", 1000);
        simpleSourceFunction = new RandomSource(sendInterval);
        break;
      default:
        logger.error("Unsupported source type: " + sourceType);
        System.exit(1);
    }

    // Prepare stream sink function
    String sinkType = params.getRequired("sink");
    SinkFunction<String> sinkFunction = null;
    SinkFunction<Row> jsonSinkFunction = null;
    SinkFunction<Record> drsSinkFunction = null;
    switch (sinkType) {
      case "hdfs":
        String basePath = params.getRequired("basePath");
        long rolloverInterval = params.getLong("rolloverInterval", 60000);
        sinkFunction = new HdfsSink(
          basePath,
          userPrincipal,
          rolloverInterval);
        env.enableCheckpointing(5000);
        break;
      case "hbase":
        sinkFunction = new HBaseSink(userPrincipal);
        break;
      case "kafka":
        String sinkTopic = params.getRequired("sink.topic");
        Properties kafkaSinkProps = new Properties();
        kafkaSinkProps.setProperty("bootstrap.servers", params.getRequired("bootstrap.servers"));
        kafkaSinkProps.setProperty("security.protocol", "SASL_PLAINTEXT");
        kafkaSinkProps.setProperty("sasl.kerberos.service.name", "kafka");
        kafkaSinkProps.setProperty("kerberos.domain.name", params.getRequired("kerberos.domain.name"));
        sinkFunction = new KafkaSink<>(sinkTopic, new SimpleStringSchema(), kafkaSinkProps, userPrincipal);
        break;
      case "dcs":
        String clusterAddress = params.getRequired("cluster.address");
        String password = params.getRequired("password");
        sinkFunction = new DCSSink(clusterAddress, password);
        break;
      case "dws":
        String dbUrl = params.getRequired("db.url");
        String userName = params.getRequired("userName");
        String dwsPassword = params.getRequired("password");
        String tableName = params.getRequired("tableName");
        sinkFunction = new DWSSink(dbUrl, userName, dwsPassword, tableName);
        break;
      case "hive":
        String zkServerPrincipal = params.getRequired("zk.server.principal");
        String dbName = params.get("dbName", "");
        if ("json".equalsIgnoreCase(encode)) {
          jsonSinkFunction = new HiveRowSink(userPrincipal, zkServerPrincipal, dbName);
        } else if ("dsr_avro".equalsIgnoreCase(encode)) {
          drsSinkFunction = new HiveDsrSink(userPrincipal, zkServerPrincipal, dbName);
        } else {
          throw new IllegalArgumentException("Unsupported encode: " + encode);
        }
        break;
      default:
        logger.error("Unsupported sink type: " + sinkType);
        System.exit(1);
    }

    // generate execute graph
    if (simpleSourceFunction != null) {
      env.addSource(simpleSourceFunction, sourceName)
        .addSink(sinkFunction)
        .disableChaining();
    } else if (jsonSourceFunction != null) {
      env.addSource(jsonSourceFunction, sourceName)
        .addSink(jsonSinkFunction)
        .disableChaining();
    } else {
      env.addSource(drsSourceFunction, sourceName)
        .addSink(drsSinkFunction)
        .disableChaining();
    }

    // execute streaming job
    env.execute();
  }

}
